# -*- coding: utf-8 -*-
"""
    settings.py
    ~~~~~~~~~~~

    utilities for building files and setting default file values

    :copyright: (c) 2015 by Mek Karpeles.
    :license: see LICENSE for more details.
"""

import datetime
import textwrap
from jinja2 import Environment, PackageLoader

env = Environment(loader=PackageLoader('pypc', 'templates'))

def license(**options):
    return env.get_template('LICENSE').render()

def readme(pkgname, desc="", **options):
    underline = "=" * len(pkgname)
    return env.get_template('README').render(
        pkgname=pkgname, underline=underline, desc=desc
        )

def manifest(**options):
    return env.get_template('MANIFEST.in').render()

def setup(pkgname, version="", desc="", url="", author="", email="",
          dependencies=None, classifiers=None, readme="README.rst", **kwargs):
    """Generates a project's setup.py"""
    return env.get_template('setup.html').render(
        name=pkgname, version=version, url=url, author=author, email=email,
        desc=desc, readme=readme, dependencies=dependencies,
        classifiers=classifiers
        )

def changelog(version, **kwargs):
    """Generates a project's CHANGES changelog"""
    return env.get_template('CHANGES').render(
        version=version, date=datetime.datetime.now().ctime()
        )

def init(pkgname, version="", desc="", author="", **kwargs):
    """
    Generates a main __init__.py for the project w/ __version__, etc.

    usage:
        >>> init("python-mypkg", description="test desc", author="mek")
    """
    return env.get_template('__init__.py').render(
        title=pkgname, desc=desc, version=version, author=author
        )

def header(name, desc, author, python, encoding):
    """Creates headers for Python source files"""
    year = datetime.date.today().year
    underline = len(name) * '~'
    desc = textwrap.fill(desc, width=79)
    return env.get_template('header.html').render(
        encoding=encoding, name=name, underline=underline, desc=desc,
        year=year, author=author
        )

def setup_opts(minimal=False, **options):
    """Fills in the appropriate defaults within options dict"""
    for k in DEFAULTS:
        options.setdefault(k, options.get(k, DEFAULTS[k]))
    options['dependencies'] = {} if minimal else options['dependencies']
    return options

def setup_fs(minimal=False, **options):
    """Return either a minimal or standard file system dict"""
    fs = MINIMAL(**options)
    if not minimal:
        fs.update(STANDARD(**options))
    return fs

MINIMAL = lambda **options: {
    options['readme']: readme(**options),
    'setup.py': setup(**options),
    'setup.cfg': env.get_template('setup.cfg').render(),
    '$': { # pkg dir
        '__init__.py': init(**options),
        }
    }

STANDARD = lambda **options: {
    'docs': {},
    'examples': {},
    'AUTHORS': "",
    'CHANGES': changelog(**options),
    'LICENSE': license(**options),
    'MANIFEST.in': manifest(**options),
    'tox.in': "",
    'tests': {
        "__init__.py": "", #todo
        }
    }

DEFAULTS = {
    'readme': 'README.rst',
    'pkgname': 'python-mypkg',
    'version': '0.0.1',
    'python': '3.4', #use env?
    'encoding': 'utf-8',
    'desc': '',
    'author': 'Anonymous',
    'email': '',
    'url': '',
    'dependencies': {
        'virtualenv': '1.11.6',
        'pep8': '1.5.7',
        'pyflakes': '0.8.1'
        },
    'classifiers': [],
    'venv': 'venv'
    }



#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    pypc
    ~~~~

    This module creates new pypc packages

    :copyright: (c) 2015 by Mek Karpeles.
    :license: see LICENSE for more details.
"""

import os
import pip
import virtualenv
import subprocess
from pkg_resources import WorkingSet, DistributionNotFound
from .settings import DEFAULTS, setup_fs, setup_opts, header

class Context(object):
    """Switches a scope's context so that it's temporarily run within
    a specified path.

    usage:
        >>> with Context('/tmp/foo') as ctx:
        ...     print("helloworld from /tmp/foo")
    """

    def __init__(self, path, back=None):
        """Context stores a path to enter and a path to return to
        after it falls outside of scope
        """
        self.back = back or os.getcwd()
        self.forward = path

    def __enter__(self):
        os.chdir(self.forward)
        return self

    def __exit__(self, type_, value, traceback):
        os.chdir(self.back)


class Package(object):
    """A class which offers utilities for generating and maintaining
    python packages (across multiple python versions)
    """

    def __init__(self, pkgname, path=None, venv='venv'):
        """Assumes path of current working directory $PWD by default.
        >>> Package('pythagorean')
        """
        _path = path or os.getcwd()
        if not os.path.exists(_path):
            raise OSError('Valid existing path required')
        if not pkgname:
            raise TypeError("Path '%s' must include a basename" % _path)
        self.path = _path
        self.pkgname = pkgname
        self.venv = venv

    def as_package(func):
        """Decorator which preempts func by switching directories to
        the Package's local context (venv) prior to execution and then
        restores back to pypc's native context afterwards.
        """
        def switch_ctx(self, *args, **kwargs):
            """Allows Package methods to easily operate within / change paths by
            wrapping them with Context.
            """
            with Context(self.path):
                return func(self, *args, **kwargs)
        return switch_ctx

    def disallow_customfs(f):
        """Convenience decorator, raises exception if user attempts to
        provide custom `fs` before feature is implemented
        """
        def inner(self, *args, **kwargs):
            if kwargs.get('fs', None):
                raise NotImplementedError
            return f(self, *args, **kwargs)
        return inner

    def strict(f):
        """Decorates func with ability to install pip dependencies +
        setup virtuenv if strict-mode (--strict flag is present)
        """
        def inner(self, *args, **kwargs):
            res = f(self, *args, **kwargs)
            if kwargs.get('strict', False):
                self.install_virtualenv()
                print('\nTo activate virtualenv, run:')
                print('\n\t`source venv/bin/activate`\n')
                pkgs = pkgs=kwargs.get('dependencies', {})
                self.install_requirements(pkgs=pkgs)
            return res
        return inner

    @strict
    @disallow_customfs
    def new(self, fs=None, **options):
        """Scaffolds a new package, creates the directory hierarchy
        """
        minimal = options.pop('minimal', False)
        strict = options.pop('strict', False)
        opts = setup_opts(minimal=minimal, **options)
        _fs = fs or setup_fs(minimal=minimal, **opts)

        def buildfs(fs, path=""):
            """Recurses the filesystem `fs` dict and touches necessary
            files with appropriate contents
            """
            if type(fs) is not dict:
                return self.touch(fs, path, **opts)

            for f in fs:
                cwd = os.path.join(path, f.replace('$', self.pkgname))
                if type(fs[f]) is dict:
                    if not os.path.exists(cwd):
                        os.makedirs(cwd)
                    buildfs(fs[f], path=cwd)
                else:
                    self.touch(fs[f], path=cwd, **opts)
        return buildfs(fs=_fs, path=self.path)

    def touch(self, contents, path, **options):
        """Touches a file or resource named $fname into existence at
        location $path and fill it with $contents data as specified

        params:
            contents - contents of file to create
            path - current full path in fs tree
        """
        if contents and not os.path.exists(path):
            fname = path.rsplit(os.sep, 1)[-1]
            if fname.endswith(".py"):
                _header = header(fname, options['desc'], options['author'],
                                 options['encoding'], options['python'])
                contents = _header + contents
            with open(path, 'w') as fout:
                fout.write(contents)

    @as_package
    def install_virtualenv(self):
        """Installs virtualenv system-wide and configures Package's
        virtual environment.
        """
        if not self.is_installed('virtualenv'):
            pip.main(['install', 'virtualenv'])
        venv = os.path.join(self.path, self.venv)
        virtualenv.create_environment(self.venv)

    @as_package
    def as_venv(self, cmd=None):
        """Runs bash cmds within the virtualenv context

        usage:
            >>> p = Package('/tmp/foo')
            >>> p.new()
            >>> p.as_venv('pip freeze')
        """
        activate = '. %s' % os.path.join(self.venv, 'bin', 'activate')
        command = '%s && %s' % (activate, cmd) if cmd else activate
        proc = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
        return proc.communicate()[0]

    @as_package
    def install_requirements(self, pkgs=None):
        """Installs default dependencies + user specified pkgs and
        then populates a requirements.txt.
        """                
        for pkg in (pkgs or DEFAULTS['dependencies']):
            self.as_venv('pip install %s' % pkg)

        # record installed packages
        with open(os.path.join(self.path, 'requirements.txt'), 'wb') as f:
            f.write(self.as_venv('pip freeze > requirements.txt'))

    @staticmethod
    def is_installed(key):
        """Determines of a package name is installed. It is
        recommended this method be run within a context decorated by
        @as_package.
        """
        return key in [d.key for d in pip.get_installed_distributions()]

    def freeze(self):
        """Returns a list of modules installed for this Package"""
        self._freeze(self.path)

    @staticmethod
    def _freeze(path):
        """Generates a list of modules installed within path's local
        context, useful for generating requirements.txt
        
        TODO Consider using context manager instead of chdir
        see: http://stackoverflow.com/a/431747
        """
        with Context(path) as ctx:
            pkgs = pip.get_installed_distributions()
            return sorted('%s==%s' % (p.key, p.version) for p in pkgs)

    def setup_virtualenv(self, name='venv'):
        """Performs the initial configuration and package environment setup"""
        try:
            dep = WorkingSet().require('virtualenv')
        except DistributionNotFound:            
            pip.main(['install', 'virtualenv'])
        self.activate_virtualenv(named=name)

    @as_package
    def activate_virtualenv(self, named):
        shell = os.environ["SHELL"]
        subprocess.Popen('mkvirtualenv %s' % named, executable=shell, shell=True,
                         stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

    @as_package
    def install(self, dependencies=None):
        """Installs base dependencies and writes pip freeze to
        requirements.txt
        """
        for d in dependencies:
            pip.main(['install', d])
        with open('requirements.txt', 'wb') as f:
            requirements = '\n'.join(self.freeze())
            f.write(requirements)



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import sys
import argparse
import getpass # experimental
from . import __title__, __version__, Package

def argparser():
    """Creates a command line ArgumentParser for pypc.
    """
    parser = argparse.ArgumentParser(description=__title__)
    parser.add_argument('-v', help="Displays the pypc version",
                        action="version", version="%s v%s" \
                            % (__title__, __version__))
    parser.add_argument('-m', '--minimal', help="Minimal mode, setup.py only",
                        action="store_true")
    parser.add_argument('--strict', dest="strict", help="Install linters",
                        action="store_true")
    parser.add_argument('--venv', dest="venv", help="Configure virtualenv",
                        default='venv')
    parser.add_argument(dest="pkgname", help="Desired package name")
    parser.add_argument('--path', dest="path", help="Path to package directory")
    parser.add_argument('--author', dest="author", help="Author's name",
                        default=getpass.getuser())
    parser.add_argument('--email', dest="email", help="Author's email",
                        default='')
    parser.add_argument('-V', '--version', dest="version",
                        help="Your package's version: defaults to v0.0.1" \
                            "(use -V for pypc version)",
                        default='0.0.1')
    parser.add_argument('--desc', dest="desc", help="Package desc",
                        default='no description available')
    parser.add_argument('--url', dest="url", help="Package url",
                        default='')
    parser.add_argument('--rm', dest="readme", help="README filename",
                        default='README.rst')
    # It'd be nice to collect analytics on how often `fs` is req'd.
    # Cookiecutter + Donald Stufft lead me to believe often
    # see: groups.google.com/forum/#!topic/pypa-dev/eaku1xvUVHU
    parser.add_argument('--fs', dest="fs", help=".json File Structure",
                        default=None)
    return parser

def optparser(args):
    """Constructs a dictionary of settings from the values the user
    provides to the ArgumentParser. Used in Package.new() and for
    """
    return {
        'venv': args.venv,
        'strict': args.strict,
        'pkgname' : args.pkgname,
        'author': args.author,
        'email': args.email,
        'version': args.version,
        'classifiers': [], #todo        
        'url': args.url,
        'desc': args.desc,
        'readme': args.readme,
        }

def main():
    parser = argparser()
    args = parser.parse_args()
    opts = optparser(args)
    pkg = Package(args.pkgname, path=args.path, venv=args.venv)
    pkg.new(**opts)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
     ____  _  _  ____   ___ 
    (  _ \( \/ )(  _ \ / __)
     ) __/ )  /  ) __/( (__ 
    (__)  (__/  (__)   \___)
    ~~~~~~~~~~~~~~~~~~~~~~~~

    pypc, the python package creator

    :copyright: (c) 2015 by Mek Karpeles.
    :license: see LICENSE for more details.
"""

__title__ = 'pypc'
__version__ = "0.1.40"
__author__ = [
    "Mek <michael.karpeles@gmail.com>"
]

import sys
from .create import Package
from .settings import DEFAULTS
from .cli import main

if __name__ == '__main__':
    sys.exit(main())



#-*- coding: utf-8 -*-

"""
pypc
----

Pypc is a tool for creating Python Packages. 

Generating a Project
````````````````````

.. code:: bash

    $ pypc helloworld

Setup
`````

.. code:: bash

    $ pip install pypc
    $ pypc helloworld

    AUTHORS CHANGES docs/ examples/ LICENSE MANIFEST.in project/ README.md requirements.txt setup.py tox.in venv/

Links
`````

* `website <http://github.com/mekarpeles/pypc/>`_

"""

import codecs
import os
import re
from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))

def read(*parts):
    """Taken from pypa pip setup.py:
    intentionally *not* adding an encoding option to open, See:
       https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    """
    return codecs.open(os.path.join(here, *parts), 'r').read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

setup(
    name='pypc',
    version=find_version("pypc", "__init__.py"),
    description="Python3 Package Creator",
    long_description=read('README.rst'),
    author='Mek Karpeles',
    author_email='michael.karpeles@gmail.com',
    url='http://github.com/mekarpeles/pypc',
    packages=[
        'pypc',
        'pypc/templates'
        ],
    platforms='any',
    include_package_data=True,
    license='LICENSE',
    install_requires=[
        'jinja2',
        'argparse',
        'flake8',
        'pyflakes',
        'virtualenv'
    ],
    classifiers=[
        'Development Status :: 1 - Planning',
        'License :: OSI Approved :: BSD License',
        "Operating System :: OS Independent",
        'Intended Audience :: Developers',
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4"
        ],
    entry_points={
        'console_scripts': ['pypc=pypc.cli:main'],
        }
)
